<?php

namespace Drupal\Tests\views\Functional;

use Drupal\language\Entity\ConfigurableLanguage;
use Drupal\views\Views;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class ViewTranslationTest.
 *
 * @group views
 */
class ViewTranslationTest extends ViewTestBase {

  /**
   * {@inheritdoc}
   */
  public static $testViews = ['test_view'];

  /**
   * {@inheritdoc}
   */
  public static $modules = ['locale', 'language', 'config_translation', 'views'];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  public function setUp($import_test_views = TRUE) {
    parent::setUp($import_test_views);

    $this->enableViewsTestModule();

    ConfigurableLanguage::createFromLangcode('ro')->save();
    $this->rebuildContainer();
  }

  /**
   * Tests that the view route title is translated.
   */
  public function testViewRouteTranslation() {
    $view = Views::getView('test_view');

    // Create a test display, add path and default language title.
    $view->storage->addDisplay('page');
    $displays = $view->storage->get('display');
    $displays['default']['display_options']['title'] = 'Title EN';
    $displays['page_1']['display_options']['path'] = 'test-view';
    $view->storage->set('display', $displays);
    $view->save();

    // We need to rebuild the routes to discover the route to the
    // view display.
    \Drupal::service('router.builder')->rebuild();

    /** @var \Drupal\Core\Routing\RouteProviderInterface $routeProvider */
    $routeProvider = \Drupal::service('router.route_provider');
    $route = $routeProvider->getRouteByName('view.test_view.page_1');

    /** @var \Drupal\Core\Controller\TitleResolver $titleResolver */
    $titleResolver = \Drupal::service('title_resolver');

    $request = Request::create('/test-view');
    $request->attributes->add([
      'view_id' => $view->id(),
      'display_id' => 'page_1',
    ]);

    // First test if the routes title gets set correctly in the
    // default language.
    $this->assertEquals('Title EN', $titleResolver->getTitle($request, $route), 'The default title is in the default language.');

    // Override language for config to translate view title.
    /** @var \Drupal\Core\Language\LanguageManagerInterface $languageManager */
    $languageManager = \Drupal::languageManager();
    $languageManager->setConfigOverrideLanguage($languageManager->getLanguage('ro'));

    // Add translation for the display.
    $displays = $view->storage->get('display');
    $displays['default']['display_options']['title'] = 'Title RO';
    $view->storage->set('display', $displays);
    $view->save();

    // Add the language in which we translated to the request.
    $request->setLocale($languageManager->getLanguage('ro')->getId());

    $this->assertEquals('Title RO', $titleResolver->getTitle($request, $route), 'The route title gets translated when language set.');
  }

}
